@@ -17,8 +17,7 @@
 #
 
 import os.path
-import gtk
-import gmenu
+from gi.repository import Gtk
 
 from config import *
 
@@ -28,12 +27,12 @@ import menufilewriter
 class MenuEditorDialog:
     def __init__ (self, menu_files):
         ui_file = os.path.join (UI_DIR, "gmenu-simple-editor.ui")
-        self.ui = gtk.Builder()
+        self.ui = Gtk.Builder()
         self.ui.set_translation_domain (PACKAGE)
         self.ui.add_from_file(ui_file)
         self.window = self.ui.get_object ("menu_editor_dialog")
-        self.window.connect ("destroy", gtk.main_quit)
-        self.window.set_default_response (gtk.RESPONSE_ACCEPT)
+        self.window.connect ("destroy", self.__dialog_destroyed)
+        self.window.set_default_response (Gtk.ResponseType.ACCEPT)
         self.window.set_icon_name ("gnome-main-menu")
 
         self.help_button = self.ui.get_object ("help_button")
@@ -59,96 +58,104 @@ class MenuEditorDialog:
     def __revert_to_system_default (self, parent_iter = None):
         model = self.menu_tree_model
         
-        iter = model.iter_children (parent_iter)
-        while iter:
-            if model[iter][model.COLUMN_IS_ENTRY]:
-                model[iter][model.COLUMN_USER_VISIBLE] = model[iter][model.COLUMN_SYSTEM_VISIBLE]
+        (has_iter, iter) = model.iter_children (parent_iter)
+        while has_iter:
+            if model.get_value (iter, model.COLUMN_IS_ENTRY):
+                visible = model.get_value (iter, model.COLUMN_SYSTEM_VISIBLE)
+                model.set_value (iter, model.COLUMN_USER_VISIBLE, visible)
             else:
                 self.__revert_to_system_default (iter)
             
-            iter = model.iter_next (iter)
+            has_iter = model.iter_next (iter)
+
+    def __dialog_destroyed (self, dialog):
+        Gtk.main_quit ()
 
     def __dialog_response (self, dialog, response_id):
-        if response_id == gtk.RESPONSE_REJECT:
+        if response_id == Gtk.ResponseType.REJECT:
             self.__revert_to_system_default ()
-            iter = self.menu_tree_model.get_iter_first ()
-            while iter:
+            (has_iter, iter) = self.menu_tree_model.get_iter_first ()
+            while has_iter:
                 self.menu_file_writer.queue_sync (iter)
-                iter = self.menu_tree_model.iter_next (iter)
+                has_iter = self.menu_tree_model.iter_next (iter)
             return
         
         dialog.destroy ()
 
-    def __is_menu_tree_directory (self, model, iter):
-        return not model[iter][self.menu_tree_model.COLUMN_IS_ENTRY]
+    def __is_menu_tree_directory (self, model, iter, data):
+        return not model.get_value(iter, self.menu_tree_model.COLUMN_IS_ENTRY)
 
     def __setup_menus_tree (self):
-        self.menus_model = self.menu_tree_model.filter_new ()
-        self.menus_model.set_visible_func (self.__is_menu_tree_directory)
+        self.menus_model = self.menu_tree_model.filter_new (None)
+        self.menus_model.set_visible_func (self.__is_menu_tree_directory, None)
         self.menus_tree.set_model (self.menus_model)
         
-        self.menus_tree.get_selection ().set_mode (gtk.SELECTION_BROWSE)
+        self.menus_tree.get_selection ().set_mode (Gtk.SelectionMode.BROWSE)
         self.menus_tree.get_selection ().connect ("changed", self.__menus_selection_changed)
         self.menus_tree.set_headers_visible (False)
 
-        column = gtk.TreeViewColumn (_("Name"))
+        column = Gtk.TreeViewColumn (_("Name"))
         column.set_spacing (6)
 
-        cell = gtk.CellRendererPixbuf ()
+        cell = Gtk.CellRendererPixbuf ()
         column.pack_start (cell, False)
-        column.set_attributes (cell, pixbuf = self.menu_tree_model.COLUMN_ICON)
+        column.add_attribute (cell, 'pixbuf', self.menu_tree_model.COLUMN_ICON)
 
-        cell = gtk.CellRendererText ()
+        cell = Gtk.CellRendererText ()
         column.pack_start (cell, True)
-        column.set_attributes (cell, text = self.menu_tree_model.COLUMN_NAME)
+        column.add_attribute (cell, 'text', self.menu_tree_model.COLUMN_NAME)
                                 
         self.menus_tree.append_column (column)
 
         self.menus_tree.expand_all ()
 
     def __setup_entries_list (self):
-        self.entries_list.get_selection ().set_mode (gtk.SELECTION_SINGLE)
+        self.entries_list.get_selection ().set_mode (Gtk.SelectionMode.SINGLE)
         self.entries_list.set_headers_visible (True)
 
-        column = gtk.TreeViewColumn (_("Show"))
+        column = Gtk.TreeViewColumn (_("Show"))
         self.entries_list.append_column (column)
         
-        cell = gtk.CellRendererToggle ()
+        cell = Gtk.CellRendererToggle ()
         cell.connect ("toggled", self.__on_hide_toggled)
         column.pack_start (cell, False)
-        column.set_attributes (cell, active = self.menu_tree_model.COLUMN_USER_VISIBLE)
+        column.add_attribute (cell, 'active', self.menu_tree_model.COLUMN_USER_VISIBLE)
 
-        column = gtk.TreeViewColumn (_("Name"))
+        column = Gtk.TreeViewColumn (_("Name"))
         column.set_spacing (6)
         self.entries_list.append_column (column)
 
-        cell = gtk.CellRendererPixbuf ()
+        cell = Gtk.CellRendererPixbuf ()
         column.pack_start (cell, False)
-        column.set_attributes (cell, pixbuf = self.menu_tree_model.COLUMN_ICON)
+        column.add_attribute (cell, 'pixbuf', self.menu_tree_model.COLUMN_ICON)
 
-        cell = gtk.CellRendererText ()
+        cell = Gtk.CellRendererText ()
         column.pack_start (cell, True)
-        column.set_attributes (cell, text = self.menu_tree_model.COLUMN_NAME)
+        column.add_attribute (cell, 'text', self.menu_tree_model.COLUMN_NAME)
 
     def __on_hide_toggled (self, toggle, path):
         def toggle_value (model, iter, column):
-            model[iter][column] = not model[iter][column]
+            value = model.get_value (iter, column)
+            model.set_value (iter, column, not value)
 
-        child_path = self.entries_model.convert_path_to_child_path (path)
-        child_iter = self.menu_tree_model.get_iter (child_path)
+        real_path = Gtk.TreePath.new_from_string (path)
+        child_path = self.entries_model.convert_path_to_child_path (real_path)
+        (has_iter, child_iter) = self.menu_tree_model.get_iter (child_path)
         
+        if not has_iter:
+            return
+
         toggle_value (self.menu_tree_model, child_iter, self.menu_tree_model.COLUMN_USER_VISIBLE)
 
         self.menu_file_writer.queue_sync (child_iter)
 
     def __menus_selection_changed (self, selection):
-        (model, iter) = selection.get_selected ()
-        if not iter:
+        (is_selected, model, iter) = selection.get_selected ()
+        if not is_selected:
             self.entries_list.set_model (None)
             return
 
-        if iter:
-            iter = model.convert_iter_to_child_iter (iter)
+        iter = model.convert_iter_to_child_iter (iter)
         
         self.entries_model = self.menu_tree_model.filter_new (self.menu_tree_model.get_path (iter))
         self.entries_model.set_visible_column (self.menu_tree_model.COLUMN_IS_ENTRY)
